
# q-expansion of newform 288.8.a.d, downloaded from the LMFDB on 05 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 288
weight = 8
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [70], [92], [-3124], [1174], [12582], [15404], [-19200], [24502], [-124796], [-262394], [681358], [-1868], [162424], [1319870], [-1540364], [-1824354], [875668], [2375056], [636522], [-7785404], [-3625596], [-2485210], [7369522], [-5980434], [-16328556], [-20897612], [21264806], [-16853666], [-31011860], [-16993684], [2687934], [-36184580], [55974678], [-42695508], [-28122610], [-60860692], [-35266376], [66187518], [6802428], [-42527170], [-154152960], [-42359406], [83346814], [-41663324], [-207037660], [171080828], [191064516], [250620830], [-218953002], [-33400320], [25345426], [-70401828], [-312387074], [44499672], [156007878], [482696652], [316922014], [-491378634], [628908844], [532085358], [317092356], [689805144], [-350308070], [939926694], [1070636908], [-661889566], [-855846980], [-378355394], [-48877618], [1446214256], [1125102516], [100867718], [823702868], [-658182176], [643732710], [1516515214], [-993028330], [346831450], [1677671780], [182519438], [-1268273304], [1555978258], [-2088913852], [-1227330084], [-1543393930], [1334929802], [-2367992514], [1541664940], [1454889524], [-3146819304], [-660904228], [-965530348], [-4417153788], [-3052220160], [1465315878], [5336853870], [-1769118876], [-2910592202], [-5457705156], [-4671720626], [805986564], [5575033054], [928207484], [-1014966894], [-4415202348], [2606921790], [-889825136], [3432672730], [-1130743700], [-4726182474], [-1106807514], [3605504028], [263346700], [1038846710], [9183670716], [5298667344], [-4672162450], [-5039043444], [6847232054], [3793769666], [-10411043242], [4345591836], [7320096188], [-10406496858], [-1106054402], [12374173048], [15857153964], [-9074431738], [16734810980], [9918053928], [-177349436], [10871371662], [18551183310], [-16705289486], [16590424254], [11071412604], [-12187687458], [17280041246], [-63091948], [19879796974], [16819146908], [25122786500], [-14451971626], [-29758127648], [-12431051754], [-6445564162], [-18193369484], [32797648864], [9536594254], [-15457022898], [-13093311140], [-14304961368], [-17367697660], [-25667443200], [24444367356], [-11123197626], [23618327066], [-16478682178], [-31960712196], [-3640608354], [-32219846124], [29782527068], [-49398630986], [-46291180632], [-25632789068], [49643141814], [-36555278654], [38236078598], [-30590552028], [27253096974], [45307744328], [-4729449350], [-27428399020], [32862256118], [-4796298452], [54762333990], [-66341002060], [-62518539554], [-26085573836], [-69756288268], [9186459646], [4192775974], [-8842511664], [85182884294], [-4637123226], [22208713492], [-28498272358], [63980513280], [-90290963678], [-99385224700], [13737407716], [-36538017922], [69679072396], [6096320934], [-74973957662], [-24606928250], [-81050312834], [122958308200], [34642185534], [49937952940], [119022892670], [-9432766638], [128673650876], [-146663688666], [68244632012], [77178361492], [85874742998], [98920684996], [119619691874], [-35678710650], [77868921348], [-91879773900], [78248693056], [43834117722], [113055666628], [-85287565882], [-23773131064], [-72433902258], [-61234162162], [88670701156], [59825296134], [141246806924], [104022879876], [-88093861834], [202246969734], [68867181336], [-161440090876], [-137429403628], [-106502010650], [-185343266228], [-776428804], [247604851014], [123171413188], [-202399500664], [-194936829982], [73119711126], [-19850363700], [246653573744], [-181488778236], [123629779932], [-249381420228], [-139842007746], [-21882683218], [-271701138064], [-280179608476], [-109983464388], [-112276723636], [95414373952], [190166943510], [101092375726], [137217917616], [91737586458], [-227645557850], [49065446796], [-53702304066], [-62248001764], [-179257762434], [35142083146], [-308820436948], [-270432285780], [174522333038], [17246590126], [-315366393018], [-189901299100], [-147984749650], [-165952024058], [-99205228412], [127411798662], [70475516270], [101605234484], [174311541098], [-21614054028], [-115915922798], [239056463748], [57644752652], [-454578506818], [-58049487062], [42107064268], [226346850072], [-89418426764], [-309146035568], [-58259961738], [-153840668044], [117275850240], [-343256091182], [39377484646], [520263068876], [-26304533554], [436956801678], [140634093852], [14688290982], [511076574916], [142167487038], [248661748686], [332877974348], [-167746932738], [351677088876], [-93697661684], [-603561335126], [165736588462], [309807733108], [-449362341316], [234728445284], [347697973858], [63736826028], [664318897686], [-741591674448], [8236604822], [423797765424], [366998149214], [-293654516234], [454198983164], [171907868712], [705721961530], [-10167806964], [777586214312], [572590998898], [-530236700610], [607665209988], [-459165066982], [-210973477322], [-554733584956], [-214735733298], [-366903435422], [95868733404], [144643223172], [641502308384], [-561265614394], [-322892306234], [703389193102], [115058114332], [-228453046532], [503344463708], [-748106942068], [73180508142], [513726967502], [-212400154390], [-421040882820], [-994372121098], [303700037438], [-913100208178], [-484530545692], [535831437478], [130036441524], [-250871866242], [-654484613876], [231591857368], [907946519278], [-46449845492], [-1281728912758], [146821839062], [-1041739325028], [593090264598], [-782375696202], [-827421835728], [-849863472780], [1089186661318], [915850293368], [-1093236226874], [-197550194154], [-736807063312], [851963042444], [-926095239580], [1092908251786], [-243035845626], [-226863150676], [553274994058], [1069781626308], [-865691533180], [-411605286096], [-1343073241698], [-62042603956], [-172825225114], [673558931556], [-219219394528], [999267901538], [661611898422], [666753920490], [-586245937362], [-659616304610], [-1315281954964], [1807362901470], [-1752793570436], [-49485487720], [1704997406340], [692719133630], [114314917804], [-573338081320], [786706099874], [-750933729226], [-529017330588], [-987620507444], [2054161999736], [-1345742994054], [1093743407980], [-869973068370], [2030669911940], [414112965526], [509588094430], [725281275366], [59922781564], [1453190802654], [-1186080362098], [500798229644], [-2270334798682], [1112625477622], [1124108006412], [1454080578196], [1040422774402], [1408711156966], [-2042562722724], [1357542576204], [-1431233174294], [1247769219494], [-1465384169536], [116254896908], [1013313782], [861360308368], [1095420127918], [-2273904837658], [2212442723296], [113135780940], [54644129450], [-573257705978], [630111672948], [-2108998334522], [2710251356308], [2532063946656]]
